#include "cy_pdl.h"
#include "cyhal.h"

/* FreeRTOS header files */
#include "FreeRTOS.h"
#include "task.h"
#include "queue.h"

#define I2C_SCB SCB1

#define I2C_PORT      (P10_0_PORT)
#define I2C_SCL_NUM   (P10_0_NUM)
#define I2C_SDA_NUM   (P10_1_NUM)
#define I2C_SCL_PIN   (P10_0_SCB1_I2C_SCL)
#define I2C_SDA_PIN   (P10_1_SCB1_I2C_SDA)



/* Assign divider type and number for I2C */
#define I2C_CLOCK PCLK_SCB1_CLOCK
#define I2C_CLK_DIV_TYPE  (CY_SYSCLK_DIV_8_BIT)
#define I2C_CLK_DIV_NUM   (0U)

/* Assign I2C interrupt number and priority */
#define I2C_INTR_NUM        ((IRQn_Type) scb_1_interrupt_IRQn)
#define I2C_INTR_PRIORITY   (7UL)



/* Allocate buffers for I2C slave operation */
#define I2C_BUFFER_SIZE (8UL)
uint8_t i2cReadBuffer[I2C_BUFFER_SIZE];
uint8_t i2cWriteBuffer[I2C_BUFFER_SIZE];

/* Allocate context for I2C operation */
cy_stc_scb_i2c_context_t i2cContext;


// low power support
cy_stc_syspm_callback_params_t callback_params =
{
    .base = I2C_SCB,
    .context = &i2cContext
};

cy_stc_syspm_callback_t i2c_deep_sleep_cb =
{
	Cy_SCB_I2C_DeepSleepCallback,
	CY_SYSPM_SLEEP, //CY_SYSPM_DEEPSLEEP,
    0,
    &callback_params,
NULL,
NULL
};


void init_i2c() {

    /* Slave configuration */
    const cy_stc_scb_i2c_config_t i2cConfig =
    {
        .i2cMode   = CY_SCB_I2C_SLAVE,
        .useRxFifo = false,
        .useTxFifo = true,
        .slaveAddress     = 0x08U,
        .slaveAddressMask = 0xFEU,
        .acceptAddrInFifo = false,
        .ackGeneralAddr   = false,
        .enableWakeFromSleep = true,
        .enableDigitalFilter = false,
        .lowPhaseDutyCycle = 0U,
        .highPhaseDutyCycle = 0U,
    };

    /* Configure I2C to operate */
    (void) Cy_SCB_I2C_Init(I2C_SCB, &i2cConfig, &i2cContext);

    /* Configure write and read buffers for communication with master */
    Cy_SCB_I2C_SlaveConfigReadBuf (I2C_SCB, i2cReadBuffer,  I2C_BUFFER_SIZE, &i2cContext);


    /* Connect SCB3 I2C function to pins */
    Cy_GPIO_SetHSIOM(I2C_PORT, I2C_SCL_NUM, I2C_SCL_PIN);
    Cy_GPIO_SetHSIOM(I2C_PORT, I2C_SDA_NUM, I2C_SDA_PIN);

    /* Configure pins for I2C operation */
    Cy_GPIO_SetDrivemode(I2C_PORT, I2C_SCL_NUM, CY_GPIO_DM_OD_DRIVESLOW);
    Cy_GPIO_SetDrivemode(I2C_PORT, I2C_SDA_NUM, CY_GPIO_DM_OD_DRIVESLOW);


    /* Connect assigned divider to be a clock source for I2C */
    Cy_SysClk_PeriphAssignDivider(I2C_CLOCK, I2C_CLK_DIV_TYPE, I2C_CLK_DIV_NUM);


    /* I2C slave desired data rate is 400 kbps.
    * To support this data rate the clk_scb frequency must be in range 7.82 – 15.38 MHz.
    * Find clk_scb valid ranges in TRM section I2C sub-section Oversampling and Bit Rate.
    * For clk_peri = 50 MHz, select divider value 4 and get clk_scb = (50 MHz / 4) = 12.5 MHz.
    * This clock frequency meets requirements above.
    */
    Cy_SysClk_PeriphSetDivider   (I2C_CLK_DIV_TYPE, I2C_CLK_DIV_NUM, 3u);
    Cy_SysClk_PeriphEnableDivider(I2C_CLK_DIV_TYPE, I2C_CLK_DIV_NUM);


    /* Populate configuration structure (code specific for CM4) */
    const cy_stc_sysint_t i2cIntrConfig =
    {
        .intrSrc      = I2C_INTR_NUM,
        .intrPriority = I2C_INTR_PRIORITY,
    };
    /* Hook interrupt service routine and enable interrupt */
    (void) Cy_SysInt_Init(&i2cIntrConfig, &I2C_Isr);
    NVIC_EnableIRQ(I2C_INTR_NUM);


    // this line is added for low power mode
    /* Register a deep sleep callback for I2C block. */
    Cy_SysPm_RegisterCallback(&i2c_deep_sleep_cb);


    /* Register callback for event notification.
    * It is better to do this during initialization before I2C is enabled.
    */
    Cy_SCB_I2C_RegisterEventCallback(I2C_SCB, I2C_SlaveEventHandler, &i2cContext);


    /* Enable I2C to operate */
    Cy_SCB_I2C_Enable(I2C_SCB);

}


/* Callback implementation */
void I2C_SlaveEventHandler(uint32_t events)
{

    /* Slave sent data to master */
    if (0UL != (events & CY_SCB_I2C_SLAVE_RD_CMPLT_EVENT))
    {
        if (0UL == (events & CY_SCB_I2C_SLAVE_ERR_EVENT))
        {
            /* Read complete without errors: update buffer content */
        }
        /* Setup read buffer for the next read transaction */
        Cy_SCB_I2C_SlaveConfigReadBuf(I2C_SCB, i2cReadBuffer,  I2C_BUFFER_SIZE, &i2cContext);
    }

    /* Disable I2C Interrupt: protect code from interruption */
    NVIC_DisableIRQ(I2C_INTR_NUM);
    /* Slave received data from master */
    if (0UL != (events & CY_SCB_I2C_SLAVE_WR_CMPLT_EVENT))
    {
        if (0UL == (events & CY_SCB_I2C_SLAVE_ERR_EVENT))
        {
            /* Write complete without errors: process received data */

        	/* Sending data to a queue */
        	BaseType_t xHigherPriorityTaskWoken = pdFALSE;
            if( xQueueSendFromISR( telemetry_queue, ( void * ) i2cWriteBuffer, &xHigherPriorityTaskWoken ) != pdPASS ) {
              CY_ASSERT_L3(!0);
            }
            portYIELD_FROM_ISR( xHigherPriorityTaskWoken );

        }
        /* Setup buffer for the next write transaction */
        Cy_SCB_I2C_SlaveConfigWriteBuf(I2C_SCB, i2cWriteBuffer, I2C_BUFFER_SIZE, &i2cContext);
    }
    /* Enable I2C interrupt */
    NVIC_EnableIRQ(I2C_INTR_NUM);

    /* Ignore all other events */
}


void I2C_Isr(void)
{
    Cy_SCB_I2C_Interrupt(I2C_SCB, &i2cContext);
}
